@@ -5,10 +5,11 @@ use {
     jsonrpc_ipc_server::{RequestContext, ServerBuilder},
     jsonrpc_server_utils::tokio,
     log::*,
+    serde::{Deserialize, Serialize},
     solana_core::{
         consensus::Tower, tower_storage::TowerStorage, validator::ValidatorStartProgress,
     },
-    solana_gossip::cluster_info::ClusterInfo,
+    solana_gossip::{cluster_info::ClusterInfo, contact_info::ContactInfo},
     solana_runtime::bank_forks::BankForks,
     solana_sdk::{
         exit::Exit,
@@ -16,6 +17,7 @@ use {
         signature::{read_keypair_file, Keypair, Signer},
     },
     std::{
+        fmt::{self, Display},
         net::SocketAddr,
         path::{Path, PathBuf},
         sync::{Arc, RwLock},
@@ -44,9 +46,9 @@ pub struct AdminRpcRequestMetadata {
 impl Metadata for AdminRpcRequestMetadata {}
 
 impl AdminRpcRequestMetadata {
-    fn with_post_init<F>(&self, func: F) -> Result<()>
+    fn with_post_init<F, R>(&self, func: F) -> Result<R>
     where
-        F: FnOnce(&AdminRpcRequestMetadataPostInit) -> Result<()>,
+        F: FnOnce(&AdminRpcRequestMetadataPostInit) -> Result<R>,
     {
         if let Some(post_init) = self.post_init.read().unwrap().as_ref() {
             func(post_init)
@@ -58,6 +60,76 @@ impl AdminRpcRequestMetadata {
     }
 }
 
+#[derive(Debug, Deserialize, Serialize)]
+pub struct AdminRpcContactInfo {
+    pub id: String,
+    pub gossip: SocketAddr,
+    pub tvu: SocketAddr,
+    pub tvu_forwards: SocketAddr,
+    pub repair: SocketAddr,
+    pub tpu: SocketAddr,
+    pub tpu_forwards: SocketAddr,
+    pub tpu_vote: SocketAddr,
+    pub rpc: SocketAddr,
+    pub rpc_pubsub: SocketAddr,
+    pub serve_repair: SocketAddr,
+    pub last_updated_timestamp: u64,
+    pub shred_version: u16,
+}
+
+impl From<ContactInfo> for AdminRpcContactInfo {
+    fn from(contact_info: ContactInfo) -> Self {
+        let ContactInfo {
+            id,
+            gossip,
+            tvu,
+            tvu_forwards,
+            repair,
+            tpu,
+            tpu_forwards,
+            tpu_vote,
+            rpc,
+            rpc_pubsub,
+            serve_repair,
+            wallclock,
+            shred_version,
+        } = contact_info;
+        Self {
+            id: id.to_string(),
+            last_updated_timestamp: wallclock,
+            gossip,
+            tvu,
+            tvu_forwards,
+            repair,
+            tpu,
+            tpu_forwards,
+            tpu_vote,
+            rpc,
+            rpc_pubsub,
+            serve_repair,
+            shred_version,
+        }
+    }
+}
+
+impl Display for AdminRpcContactInfo {
+    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
+        writeln!(f, "Identity: {}", self.id)?;
+        writeln!(f, "Gossip: {}", self.gossip)?;
+        writeln!(f, "TVU: {}", self.tvu)?;
+        writeln!(f, "TVU Forwards: {}", self.tvu_forwards)?;
+        writeln!(f, "Repair: {}", self.repair)?;
+        writeln!(f, "TPU: {}", self.tpu)?;
+        writeln!(f, "TPU Forwards: {}", self.tpu_forwards)?;
+        writeln!(f, "TPU Votes: {}", self.tpu_vote)?;
+        writeln!(f, "RPC: {}", self.rpc)?;
+        writeln!(f, "RPC Pubsub: {}", self.rpc_pubsub)?;
+        writeln!(f, "Serve Repair: {}", self.serve_repair)?;
+        writeln!(f, "Last Updated Timestamp: {}", self.last_updated_timestamp)?;
+        writeln!(f, "Shred Version: {}", self.shred_version)
+    }
+}
+
 #[rpc]
 pub trait AdminRpc {
     type Metadata;
@@ -90,6 +162,9 @@ pub trait AdminRpc {
         keypair_file: String,
         require_tower: bool,
     ) -> Result<()>;
+
+    #[rpc(meta, name = "contactInfo")]
+    fn contact_info(&self, meta: Self::Metadata) -> Result<AdminRpcContactInfo>;
 }
 
 pub struct AdminRpcImpl;
@@ -182,14 +257,40 @@ impl AdminRpc for AdminRpcImpl {
         })?;
 
         meta.with_post_init(|post_init| {
-            if require_tower {
-                let _ = Tower::restore(meta.tower_storage.as_ref(), &identity_keypair.pubkey())
-                    .map_err(|err| {
+            // Ensure a Tower exists for the new identity and exit gracefully.
+            // ReplayStage will be less forgiving if it fails to load the new tower.
+            if let Err(err) =
+                Tower::restore(meta.tower_storage.as_ref(), &identity_keypair.pubkey()).map_err(
+                    |err| {
                         jsonrpc_core::error::Error::invalid_params(format!(
                             "Unable to load tower file for identity {}: {}",
                             identity_keypair.pubkey(),
                             err
                         ))
+                    },
+                )
+            {
+                if require_tower {
+                    return Err(err);
+                }
+
+                let root_bank = post_init.bank_forks.read().unwrap().root_bank();
+                let mut tower = Tower::new(
+                    &identity_keypair.pubkey(),
+                    &post_init.vote_account,
+                    root_bank.slot(),
+                    &root_bank,
+                );
+                // Forge a single vote to pacify `Tower::adjust_lockouts_after_replay` when its called
+                // by replay_stage
+                tower.record_bank_vote(&root_bank, &post_init.vote_account);
+                tower
+                    .save(meta.tower_storage.as_ref(), &identity_keypair)
+                    .map_err(|err| {
+                        jsonrpc_core::error::Error::invalid_params(format!(
+                            "Unable to create default tower file for ephemeral identity: {}",
+                            err
+                        ))
                     })?;
             }
 
@@ -201,6 +302,10 @@ impl AdminRpc for AdminRpcImpl {
             Ok(())
         })
     }
+
+    fn contact_info(&self, meta: Self::Metadata) -> Result<AdminRpcContactInfo> {
+        meta.with_post_init(|post_init| Ok(post_init.cluster_info.my_contact_info().into()))
+    }
 }
 
 // Start the Admin RPC interface
