@@ -1,5 +1,6 @@
 use {
     clap::{crate_name, value_t, value_t_or_exit, values_t_or_exit, App, Arg},
+    crossbeam_channel::unbounded,
     log::*,
     solana_clap_utils::{
         input_parsers::{pubkey_of, pubkeys_of, value_of},
@@ -35,7 +36,7 @@ use {
         net::{IpAddr, Ipv4Addr, SocketAddr},
         path::{Path, PathBuf},
         process::exit,
-        sync::{mpsc::channel, Arc, RwLock},
+        sync::{Arc, RwLock},
         time::{Duration, SystemTime, UNIX_EPOCH},
     },
 };
@@ -311,14 +312,12 @@ fn main() {
                 ),
         )
         .arg(
-            Arg::with_name("geyser_plugin_config")
-                .long("geyser-plugin-config")
-                .alias("accountsdb-plugin-config")
+            Arg::with_name("accountsdb_plugin_config")
+                .long("accountsdb-plugin-config")
                 .value_name("FILE")
                 .takes_value(true)
                 .multiple(true)
-                .hidden(true)
-                .help("Specify the configuration file for the Geyser plugin."),
+                .help("Specify the configuration file for the AccountsDb plugin."),
         )
         .arg(
             Arg::with_name("no_accounts_db_caching")
@@ -550,7 +549,7 @@ fn main() {
     let faucet_pubkey = faucet_keypair.pubkey();
 
     if let Some(faucet_addr) = &faucet_addr {
-        let (sender, receiver) = channel();
+        let (sender, receiver) = unbounded();
         run_local_faucet_with_port(faucet_keypair, sender, None, faucet_addr.port());
         let _ = receiver.recv().expect("run faucet").unwrap_or_else(|err| {
             println!("Error: failed to start faucet: {}", err);
@@ -684,9 +683,9 @@ fn main() {
         genesis.bind_ip_addr(bind_address);
     }
 
-    if matches.is_present("geyser_plugin_config") {
-        genesis.geyser_plugin_config_files = Some(
-            values_t_or_exit!(matches, "geyser_plugin_config", String)
+    if matches.is_present("accountsdb_plugin_config") {
+        genesis.accountsdb_plugin_config_files = Some(
+            values_t_or_exit!(matches, "accountsdb_plugin_config", String)
                 .into_iter()
                 .map(PathBuf::from)
                 .collect(),
